Attribute VB_Name = "modPetersMap"
Option Explicit

Public MapWidth As Integer
Public MapFactor As Double
Public MapHeight As Double

'Ellipsoid Data
Public d_a As Double     'semi-major axis
Public d_b As Double     'semi-minor axis
Public d_e As Double     'numeric eccentricity
Public d_f As Double     'flattening

Public Const Pi As Double = 3.14159265358979
Public iScale As Integer 'scale (width / 360)

'Convert geographic to geozentric latitude
Public Function Phi2Psi(d_Phi As Double, d_e As Double) As Double
   'Parameters:
   'd_phi = geographic latitude (degrees)
   'd_e   = numeric eccentricity
   Phi2Psi = Atn((1 - d_e ^ 2) * Tan(Deg2Rad(d_Phi)))
   Phi2Psi = Rad2Deg(Phi2Psi)
   'Return geocentric latitude (degrees)
End Function

Public Function Deg2Rad(Deg As Double) As Double
   'convert degrees to radian
   Deg2Rad = Deg * (Pi / 180)
End Function

Public Function Rad2Deg(Rad As Double) As Double
   'convert radian to degrees
   Rad2Deg = Rad * (180 / Pi)
End Function

Function Lng2PetersX(ByVal Lambda As Double) As Double
   'central meridian set to East 11.4
   Dim Lambda0      As Double
   Dim LambdaX      As Double
   Dim dScaleFactor As Double
   Dim dLeftEdge    As Double
   dScaleFactor = MapWidth / 360
   Lambda0 = 11.4
   If Lambda < -168.6 Then Lambda = 360 + Lambda
   LambdaX = Lambda - Lambda0
   dLeftEdge = (-168.6 - Lambda0) * -1
   Lng2PetersX = (LambdaX + dLeftEdge) * dScaleFactor
End Function

Function Lat2PetersY(ByVal Phi As Double) As Double
   Dim Psi As Double 'geozentric latitide
   Dim d_z As Double   'sheroidzone
   Dim d_r As Double   '?
   Dim d_h As Double   'height
   Dim Zone0      As Double  'area of ellipsoidzone between equator and phi = 1N
   Dim ZoneY      As Double  'area of ellipsoidzone between equator and desired Phi
   Dim Segment0   As Double  'area of zonesegment (Zone0 / 180)
   Dim Baseline   As Double  'length of baseline
   Dim CoordFactor As Double
   Zone0 = PetersF(d_b, d_e, Phi2Psi(1, d_e))
   Segment0 = Zone0 / 180
   Baseline = Sqr(Segment0)
   CoordFactor = Zone0 / Baseline
   ZoneY = PetersF(d_b, d_e, Phi2Psi(Phi, d_e))
   Lat2PetersY = ZoneY / CoordFactor * MapFactor
End Function

Public Function PetersF(ByVal b As Double, ByVal e As Double, ByVal PsiD As Double) As Double
   'Parameters:
   'b    = semi-minor axis
   'e    = numeric eccentricity
   'PsiD = geozentric latitude (degrees)
   Dim PsiR As Double 'geocentric latitude (radians)
   PsiR = Deg2Rad(PsiD)
   PetersF = b ^ 2 * Pi * (((Sin(PsiR) * Sqr(1 - e ^ 2 * Cos(PsiR) ^ 2 * (2 - e ^ 2))) / ((1 - e ^ 2) * (1 - e ^ 2 * Cos(PsiR) ^ 2))) + (1 / e) * Log(((e * Sin(PsiR)) + Sqr(1 - e ^ 2 * Cos(PsiR) ^ 2 * (2 - e ^ 2))) / (Sqr(1 - e ^ 2) * Sqr(1 - e ^ 2 * Cos(PsiR) ^ 2))))
End Function

